<?php

/**
 * @package  fabrik
 * @version 906 | rob | 2008-05-20 11:06:14 +0200 (Tue, 20 May 2008
 * @Copyright (C) Rob Clayburn
 * @license GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

/* MOS Intruder Alerts */
defined( '_VALID_MOS' ) or die( 'Direct Access to this location is not allowed.' );

class fabrik_import extends mosInstaller{
	
	
/**
* @param string The name of the php (temporary) uploaded file
* @param string The name of the file to put in the temp directory
* @param string The message to return
*/
function uploadFile( $filename, $userfile_name, &$msg ) {
	global $mosConfig_absolute_path;
	$baseDir = mosPathName( $mosConfig_absolute_path . '/media' );

	if (file_exists( $baseDir )) {
		if (is_writable( $baseDir )) {
			if (move_uploaded_file( $filename, $baseDir . $userfile_name )) {
				if (mosChmod( $baseDir . $userfile_name )) {
					return true;
				} else {
					$msg = 'Failed to change the permissions of the uploaded file.';
				}
			} else {
				$msg = 'Failed to move uploaded file to <code>/media</code> directory.';
			}
		} else {
			$msg = 'Upload failed as <code>/media</code> directory is not writable.';
		}
	} else {
		$msg = 'Upload failed as <code>/media</code> directory does not exist.';
	}
	return false;
}
	
}

class fabrik_csv_import extends fabrik_import{
	
	var $headings = null;
	var $data = null;
	
	/**
	 * checks uploaded file, and installs it
	 */

	function checkUpload(){
		if (!(bool)ini_get('file_uploads')) {
			echo "The installer can't continue before file uploads are enabled. Please use the install from directory method.";
			return false;
		}
		$userfile = mosGetParam( $_FILES, 'userfile', null );
		if (!$userfile) {
			echo 'No file selected<br />';
			return false;
		}
		$userfile_name = $userfile['name'];
		$msg = '';
		$resultdir = $this->uploadFile( $userfile['tmp_name'], $userfile['name'], $msg );
		if ($resultdir !== false) {
			$this->readCSV($userfile['name']);
			
		} else {
			echo $msg . '  Upload Error<br />';
			return false;
		}	
		return true;
	}
	
	/**
	 * read the CSV file, store results in $this->headings and $this->data
	 */

	function readCSV( $userfile_name ){
		global $mosConfig_absolute_path, $database;
		$baseDir 			= mosPathName( $mosConfig_absolute_path . '/media' );
		$file_handle 		= fopen($baseDir . '/' . $userfile_name, "r");
		$this->headings 	= array();
		$this->data 		= array();
		$field_delimiter 	= mosGetParam( $_POST, 'field_delimiter', ',' );
		$text_delimiter 	= stripslashes(mosGetParam( $_POST, 'text_delimiter', '"' ));
		$table 				= mosGetParam( $_POST, 'table_name', '' );
		$csv				 = & new csv_bv($baseDir . '/' . $userfile_name, $field_delimiter, $text_delimiter, '\\');
		$csv->SkipEmptyRows(TRUE); // Will skip empty rows. TRUE by default. (Shown here for example only).
		$csv->TrimFields(TRUE); // Remove leading and trailing \s and \t. TRUE by default.

		while ($arr_data = $csv->NextLine()){
			if(empty($this->headings)){
				$this->headings =  $arr_data;
			}else{
				$this->data[] = $arr_data;
			}
		 }
		fclose($file_handle);
	}
}

/********************** */

/**
 * This class will parse a csv file in either standard or MS Excel format.
 * Two methods are provided to either process a line at a time or return the whole csv file as an array.
 *
 * It can deal with:
 * - Line breaks within quoted fields
 * - Character seperator (usually a comma or semicolon) in quoted fields
 * - Can leave or remove leading and trailing \s or \t
 * - Can leave or skip empty rows.
 * - Windows and Unix line breaks dealt with automatically. Care must be taken with Macintosh format.
 *
 * Also, the escape character is automatically removed.
 *
 * NOTICE:
 * - Quote character can be escaped by itself or by using an escape character, within a quoted field (i.e. "" or \" will work)
 *
 * USAGE:
 *
 * include_once 'class.csv_bv.php';
 *
 * $csv = & new csv_bv('test.csv', ';', '"' , '\\');
 * $csv->SkipEmptyRows(TRUE); // Will skip empty rows. TRUE by default. (Shown here for example only).
 * $csv->TrimFields(TRUE); // Remove leading and trailing \s and \t. TRUE by default.
 *
 * while ($arr_data = $csv->NextLine()){
 *
 *         echo "<br><br>Processing line ". $csv->RowCount() . "<br>";
 *         echo implode(' , ', $arr_data);
 *
 * }
 *
 * echo "<br><br>Number of returned rows: ".$csv->RowCount();
 * echo "<br><br>Number of skipped rows: ".$csv->SkippedRowCount();
 *
 * ----
 * OR using the csv2array function.
 * ----
 *
 * include_once 'class.csv_bv.php';
 *
 * $csv = & new csv_bv('test.csv', ';', '"' , '\\');
 * $csv->SkipEmptyRows(TRUE); // Will skip empty rows. TRUE by default. (Shown here for example only).
 * $csv->TrimFields(TRUE); // Remove leading and trailing \s and \t. TRUE by default.
 *
 * $_arr = $csv->csv2Array();
 * print_r($_arr);
 *
 * echo "<br><br>Number of returned rows: ".$csv->RowCount();
 * echo "<br><br>Number of skipped rows: ".$csv->SkippedRowCount();
 *
 *
 * WARNING:
 * - Macintosh line breaks need to be dealt with carefully. See the PHP help files for the function 'fgetcsv'
 *
 * The coding standards used in this file can be found here: http://www.dagbladet.no/development/phpcodingstandard/
 *
 *    All commets and suggestions are welcomed.
 *
 * SUPPORT: Visit http://vhd.com.au/forum/
 *
 * CHANGELOG:
 *
 * - Fixed skipping of last row if the last row did not have a new line. Thanks to Florian Bruch and Henry Flurry. (2006_05_15)
 * - Changed the class name to csv_bv for consistency. (2006_05_15)
 * - Fixed small problem where line breaks at the end of file returned a warning (2005_10_28)
 *
 * @author Ben Vautier <classes@vhd.com.au>
 * @copyright (c) 2006
 * @license BSD
 * @version 1.2 (2006_05_15)
 */


class csv_bv
{
	/**
	 * Seperator character
	 * @var char
	 * @access private
	 */
	var $mFldSeperator;

	/**
	 * Enclose character
	 * @var char
	 * @access private
	 */
	var $mFldEnclosure;

	/**
	 * Escape character
	 * @var char
	 * @access private
	 */
	var $mFldEscapor;

	/**
	 * Length of the largest row in bytes.Default is 4096
	 * @var int
	 * @access private
	 */
	var $mRowSize;

	/**
	 * Holds the file pointer
	 * @var resource
	 * @access private
	 */
	var $mHandle;

	/**
	 * Counts the number of rows that have been returned
	 * @var int
	 * @access private
	 */
	var $mRowCount;

	/**
	 * Counts the number of empty rows that have been skipped
	 * @var int
	 * @access private
	 */
	var $mSkippedRowCount;

	/**
	 * Determines whether empty rows should be skipped or not.
	 * By default empty rows are returned.
	 * @var boolean
	 * @access private
	 */
	var $mSkipEmptyRows;

	/**
	 * Specifies whether the fields leading and trailing \s and \t should be removed
	 * By default it is TRUE.
	 * @var boolean
	 * @access private
	 */
	var $mTrimFields;

	/**
	 * Constructor
	 *
	 * Only used to initialise variables.
	 *
	 * @param str $file - file path
	 * @param str $seperator - Only one character is allowed (optional)
	 * @param str $enclose - Only one character is allowed (optional)
	 * @param str $escape - Only one character is allowed (optional)
	 * @access public
	 */
	Function csv_bv($file, $seperator = ',', $enclose = '"', $escape = ''){

		$this->mFldSeperator = $seperator;
		$this->mFldEnclosure = $enclose;
		$this->mFldEscapor = $escape;

		$this->mSkipEmptyRows = TRUE;
		$this->mTrimFields =  TRUE;

		$this->mRowCount = 0;
		$this->mSkippedRowCount = 0;

		$this->mRowSize = 4096;

		// Open file
		$this->mHandle = @fopen($file, "r") or trigger_error('Unable to open csv file', E_USER_ERROR);
	}


	/**
	 * csv::NextLine() returns an array of fields from the next csv line.
	 *
	 * The position of the file pointer is stored in PHP internals.
	 *
	 * Empty rows can be skipped
	 * Leading and trailing \s and \t can be removed from each field
	 *
	 * @access public
	 * @return array of fields
	 */
	Function NextLine(){

		if (feof($this->mHandle)){
			return False;
		}

		$arr_row = fgetcsv ($this->mHandle, $this->mRowSize, $this->mFldSeperator, $this->mFldEnclosure);

		$this->mRowCount++;

		//-------------------------
		// Skip empty rows if asked to
		if ($this->mSkipEmptyRows){


			if ($arr_row[0] === ''  && count($arr_row) === 1){

				$this->mRowCount--;
				$this->mSkippedRowCount++;

				$arr_row = $this->NextLine();

				// This is to avoid a warning when empty lines are found at the bvery end of a file.
				if (!is_array($arr_row)){ // This will only happen if we are at the end of a file.
					return FALSE;
				}
			}
		}

		//-------------------------
		// Remove leading and trailing spaces \s and \t
		if ($this->mTrimFields && is_array($arr_row)){
			array_walk($arr_row, array($this, 'ArrayTrim'));
		}

		//-------------------------
		// Remove escape character if it is not empty and different from the enclose character
		// otherwise fgetcsv removes it automatically and we don't have to worry about it.
		if ($this->mFldEscapor !== '' && $this->mFldEscapor !== $this->mFldEnclosure && is_array($arr_row)){
			array_walk($arr_row, array($this, 'ArrayRemoveEscapor'));
		}

		return $arr_row;
	}

	/**
	 * csv::Csv2Array will return the whole csv file as 2D array
	 *
	 * @access public
	 */
	Function Csv2Array(){

		$arr_csv = array();

		while ($arr_row = $this->NextLine()){
			$arr_csv[] = $arr_row;
		}

		return $arr_csv;
	}

	/**
	 * csv::ArrayTrim will remove \s and \t from an array
	 *
	 * It is called from array_walk.
	 * @access private
	 */
	Function ArrayTrim(&$item, $key){
		$item = trim($item, " \t"); // space and tab
	}

	/**
	 * csv::ArrayRemoveEscapor will escape the enclose character
	 *
	 * It is called from array_walk.
	 * @access private
	 */
	Function ArrayRemoveEscapor(&$item, $key){
		$item = str_replace($this->mFldEscapor.$this->mFldEnclosure, $this->mFldEnclosure, $item);
	}

	/**
	 * csv::RowCount return the current row count
	 *
	 * @access public
	 * @return int
	 */
	Function RowCount(){
		return $this->mRowCount;
	}

	/**
	 * csv::RowCount return the current skipped row count
	 *
	 * @access public
	 * @return int
	 */
	Function SkippedRowCount(){
		return $this->mSkippedRowCount;
	}

	/**
	 * csv::SkipEmptyRows, sets whether empty rows should be skipped or not
	 *
	 * @access public
	 * @param bool $bool
	 * @return void
	 */
	Function SkipEmptyRows($bool = TRUE){
		$this->mSkipEmptyRows = $bool;
	}

	/**
	 * csv::TrimFields, sets whether fields should have their \s and \t removed.
      *
      * @access public
      * @param bool $bool
      * @return void
      */
    Function TrimFields($bool = TRUE){
        $this->mTrimFields = $bool;
    }
    
} 

/************************/		
class fabrik_package_import extends fabrik_import{
	var $_afabrikObjs = array();
	
	function install( $p_fromdir = null ) {
		global $mosConfig_absolute_path, $database;
		@ini_set('memory_limit', '50M');
		if (!$this->preInstallCheck( $p_fromdir, 'fabrik' )) {
			return false;
		}
		$overwritetables = mosGetParam($_REQUEST, 'overwritetables', 0);
		$this->_oConn = new fabrikConnection( $database );
		
		
		$connection = mosGetParam( $_REQUEST, 'connection_id', 0);
		if( $connection == 0 ){
			$this->_oConn->loadDefaultConnection();
		} else {
			$this->_oConn->load( $connection );
		}
		$this->_fabrikDb = $this->_oConn->loadTableConnectionDB();
		
		
		if($overwritetables){
			if( $this->createDataTables() ){
				echo "<div style='color:green'>Database tables - created</div>";
			}else{
				echo "<div style='color:red'>Database tables - some errors occured</div>";
			}
		}else{
			echo "Database tables - ignored<br />";
		}
		$aKeyMap = $this->getKeyMap();
		echo "<div style='color:green'>key map created</div>";
		$this->updateObjKeys( $aKeyMap );
		echo "<div style='color:green'>primary keys updated</div>";
		if( $this->installTemplates() ){
			echo "<div style='color:red'>templates not fully installed</div>";
		}else{
			echo "<div style='color:green'>templates installed</div>";
		}
		return true;
	}
	

	/**
	 * get a map array that maps the ids in the installer file with the available ones in the installation site
	 */

	function getKeyMap(){
		$aKeyMap = array();
		$xmlDoc 	= $this->xmlDoc();
		$root =& $xmlDoc->documentElement;
		$nodes = &$root->getElementsByPath('tables/table');
    $aKeyMap = $this->xmlToObjs( $nodes, 'fabrikTable', 'tables', $aKeyMap )	;
		$nodes = &$root->getElementsByPath('forms/form');
    $aKeyMap = $this->xmlToObjs( $nodes, 'fabrikForm', 'forms', $aKeyMap );
    	
    $nodes = &$root->getElementsByPath('groups/group');
    $aKeyMap = $this->xmlToObjs( $nodes, 'fabrikGroup', 'groups', $aKeyMap );
    	
    $nodes = &$root->getElementsByPath('elements/element');
    $aKeyMap = $this->xmlToObjs( $nodes, 'fabrikElement', 'elements', $aKeyMap );

		$nodes = &$root->getElementsByPath('validations/validation');
		$aKeyMap = $this->xmlToObjs( $nodes, 'fabrikValidation', 'validations', $aKeyMap );
    	
    return $aKeyMap;
	}
	
	   /**
     * update the keys and saves the objects
     * @param array key map
     */
     
    function updateObjKeys( $aKeyMap ){
    	if( !empty( $this->_afabrikObjs ) ){
    		foreach($this->_afabrikObjs['tables'] as $obj ){
	    		
	    		$obj->form_id = $aKeyMap['forms'][$obj->form_id];
	    		$obj->connection_id = $this->_oConn->id;
	    		$obj->store();
	    	}
	    	
	    	foreach($this->_afabrikObjs['forms'] as $obj ){
	    		$obj->table_id = $aKeyMap['tables'][$obj->table_id];
	    		$obj->store();
	    	}
	    	
	    	foreach($this->_afabrikObjs['elements'] as $obj ){
	    		$obj->group_id = $aKeyMap['groups'][$obj->group_id];
	    		$obj->join_conn_id = $this->_oConn->id;
	    		$obj->store();
	    	}
	    	
	    	foreach($this->_afabrikObjs['validations'] as $obj ){
	    		$obj->element_id = $aKeyMap['elements'][$obj->element_id];
	    		$obj->store();
	    	}
    	}else{
    		echo "problems updating object keys<br/>";
    	}
    }
	
	/**
	 * create any tables used to store data
	 * @return bol success
	 */
	
	function createDataTables(){
		global $database;
		$xmlDoc 	= $this->xmlDoc();
		$root =& $xmlDoc->documentElement;
		$query_element = &$root->getElementsByPath('queries', 1);
		$err = true;
		if(!$query_element){
			return;
		}
		if (!$query_element->hasChildNodes()) {
			return;
		}
		if (!is_null($query_element)) {
			$queries = $query_element->childNodes;
			
			foreach($queries as $query){
				$this->_fabrikDb->setQuery( $query->getText());
				if (!$this->_fabrikDb->query())
				{
					echo $this->_fabrikDb->getErrorMsg( ) . "<pre>".$this->_fabrikDb->getQuery()."</pre><br />"  ;
					$err = false;
				}
			}
		}
		return $err;
		
	}
	/**
	 * copies the template files from the tgz file into the fabrik tmpl location
	 * Note: overwrites existing files with no warning
	 *@return bol true if templates installed without error
	 */

	function installTemplates(){
		global $mosConfig_absolute_path;
		$ok = true;
		$xmlDoc 	= $this->xmlDoc();
		$root =& $xmlDoc->documentElement;
		$files_element =& $root->getElementsByPath( 'files', 1 );
		if (is_null( $files_element )) {
			echo "<div style='color:red'>error with file section</div>";
			$ok = false;
		}
		
		if (!$files_element->hasChildNodes()) {
			return;
		}
		$files = $files_element->childNodes;
		
		foreach ($files as $file) {
			$filePath =  $file->getText();
			$aBits = explode('/', $filePath);
			$fileName = array_pop($aBits);
			$filePath = implode('/', $aBits);	
			$installTo =  mosPathName( $mosConfig_absolute_path . '/components/com_fabrik/' . $filePath );
			$installFrom = mosPathName( $this->installDir() . $filePath );
			$result = $this->copyFiles( $installFrom, $installTo, array($fileName), true );
			if(!$result){
				$ok = false;
			}
		}
		return $ok;
	}
	
	 /**
     * take the xml objects and convert them into fabrik objects, doesn't saves them as keys need updating
     * @param object node list
     * @param string fabrik class name to use
     * @param string fabrik element type
     * @param array key map - records old keys and new keys (not this func doesn't update the keys - this happens in updateObjKeys())
    */
     
    function xmlToObjs( $nodes, $class, $type, $aKeyMap ){
    	$this->_afabrikObjs[$type] = array();
 		global $database;
    	if ($nodes != null) {
	    	$total = $nodes->getLength();
			for ($i = 0; $i < $total; $i++) {
				$obj = new $class( $database );
			    /*get current node on list */
			    $currNode =&$nodes->item($i);
			    $pairs = $currNode->childNodes;
			    foreach( $pairs as $query ){
					$key = $query->getTagName();
					$val =  $query->getText();
					if($key != 'id' ){
						$obj->$key = $val;
					}else{
						$oldKey = $val;
					}
				}
				$obj->store();
				$this->_afabrikObjs[$type][] = $obj;
				$newKey = $database->insertid();
				
				/*we need to get the form_id and ordering attributes so we can create
				form_group record
				note: form_id is the NEW form id here */ 
				
				if($type == 'groups'){
					if ($currNode->hasAttribute("form_id")) {
						$form_id = $currNode->getAttribute("form_id");
					}
					$newFormId = $aKeyMap['forms'][$form_id];
					if ($currNode->hasAttribute("ordering")) {
						$ordering = $currNode->getAttribute("ordering");
					}
					$sql = "INSERT INTO #__fabrik_formgroup ( form_id, group_id, ordering) VALUES ('$newFormId', '$newKey', '$ordering')";
					$database->setQuery( $sql );
					$database->query( );
					$aKeyMap['formgroup'][$form_id] = $newKey;
				}
				$aKeyMap[$type][$oldKey] = $newKey;
			}
    	} else{
    		echo "$type: nodes are null<br/>";
    	}
    	return $aKeyMap; 	
    }
}
?>